from i_view_plugin import IViewPlugin, VIEW_PLUGIN_EXT
from histogram import HistogramPlugin
from hexbin import HexbinPlugin
from scatterplot import ScatterplotPlugin
from bar_chart import BarChartPlugin
from stats_1d import Stats1DPlugin